AUI.WorldMap = {}

local function AUI_WorldMap_GetCurrentMapIndex()
	return tonumber(GetMapType() .. GetMapContentType())
end

local function AUI_WorldMap_SetZoom(zoomMin, zoomMax)
	ZO_WorldMap_SetCustomZoomLevels(zoomMin, zoomMax)
	
	zo_callLater(function() ZO_WorldMap_ClearCustomZoomLevels() end, 50)
end

local function SetData(_self, _pinType, _pinTag)
	local auiPinData = nil
	
	if _self:IsQuest() then
		local questType = GetJournalQuestType(_pinTag[1])
		local questRepeatType = GetJournalQuestRepeatType(_pinTag[1]) 				
		local AUI_pinType = AUI.Minimap.Pin.GetQuestPinType(_pinType, questType, questRepeatType)
		auiPinData = AUI_MINIMAP_PIN_DATA[AUI_pinType]
	elseif _self:IsGroup() then
		local AUI_pinType = AUI.Minimap.Pin.GetGroupStatePinType(_self.m_PinTag)
		auiPinData = AUI_MINIMAP_PIN_DATA[AUI_pinType]
	elseif AUI_MINIMAP_PIN_DATA[pinType] and ZO_MapPin.PIN_DATA[_pinType] then
		local AUI_pinType = _pinType
		auiPinData = AUI_MINIMAP_PIN_DATA[AUI_pinType]
	end		
	
	if auiPinData then
		local pinControl = _self.m_Control
			
		local overlayControl = GetControl(pinControl, "Background")
		local highlightControl = GetControl(pinControl, "Highlight")
		local overlayTexture, pulseTexture, glowTexture = AUI.Minimap.Pin.GetPinTextureData(_self, auiPinData.texture)
		
		if(overlayTexture ~= "") then	
			overlayControl:SetTexture(overlayTexture)
		end		
							
		if auiPinData.tint then
			local tint = AUI.Minimap.Pin.GetPinTextureColor(self, auiPinData.tint)
			if tint then
				overlayControl:SetColor(tint:UnpackRGBA())
			end
		else
			overlayControl:SetColor(1, 1, 1, 1)
		end	

		if auiPinData.grayscale then
			local grayscale = auiPinData.grayscale		
			overlayControl:SetDesaturation((type(grayscale) == "function" and grayscale(self) or grayscale) and 1 or 0)
		else
			overlayControl:SetDesaturation(0)
		end				
					 
		if pulseTexture then
			self:ResetAnimation(RESET_ANIM_ALLOW_PLAY, LONG_LOOP_COUNT, pulseTexture, overlayTexture, AUI_Pin.DoFinalFadeInAfterPing)
		elseif glowTexture then
			highlightControl:SetHidden(false)
			highlightControl:SetAlpha(1)				
			highlightControl:SetTexture(glowTexture)
			highlightControl:ClearAnchors()
			highlightControl:SetAnchor(TOPLEFT, control, TOPLEFT, -5, -5)
			highlightControl:SetAnchor(BOTTOMRIGHT, control, BOTTOMRIGHT, 5, 5)					
		else
			highlightControl:SetHidden(true)
		end
	end
end

function AUI.WorldMap.Init()
	AUI.PreHook(ZO_MapPin, "SetData", SetData)
	
     CALLBACK_MANAGER:RegisterCallback("OnWorldMapChanged", function(wasNavigateIn)
		if wasNavigateIn == nil then
			if AUI.Settings.Minimap.worldmap_zoom_out then
				AUI_WorldMap_SetZoom(1, 1)
			end
		end
    end)
		
	WORLD_MAP_SCENE:RegisterCallback("StateChange", 
	function(oldState, newState)
		if AUI.Minimap.IsLoaded() then
			if newState == SCENE_SHOWING then					
				if AUI.Minimap.IsLocationChanged() then
					CALLBACK_MANAGER:FireCallbacks("OnWorldMapChanged")
				elseif AUI.Settings.Minimap.worldmap_zoom_out then
					AUI_WorldMap_SetZoom(1, 1)
				end
				
				ZO_WorldMap_RefreshGroupPins()
			elseif newState == SCENE_HIDDEN then
				AUI.Minimap.Show()
			end
		end													
	end)
	
	GAMEPAD_WORLD_MAP_SCENE:RegisterCallback("StateChange", 
	function(oldState, newState)
		if AUI.Minimap.IsLoaded() then
			if newState == SCENE_SHOWING then					
				if AUI.Minimap.IsLocationChanged() then
					CALLBACK_MANAGER:FireCallbacks("OnWorldMapChanged")
				elseif AUI.Settings.Minimap.worldmap_zoom_out then
					AUI_WorldMap_SetZoom(1, 1)
				end	
			elseif newState == SCENE_HIDDEN then
				AUI.Minimap.Show()
			end
		end													
	end)	
end